@@ -3,25 +3,13 @@
 """
 Copyright (C) 2008-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
-Kajongg is free software you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation either version 2 of the License, or
-(at your option) any later version.
+SPDX-License-Identifier: GPL-2.0
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """
 
 # pylint: disable=wrong-import-order, wrong-import-position
 
 import sys
-import os
 import codecs
 from itertools import chain
 
@@ -29,10 +17,10 @@ import cgitb
 import tempfile
 import webbrowser
 import logging
-from signal import signal, SIGABRT, SIGINT, SIGTERM
 
 from log import logError, logDebug
-from common import Options, Internal, isAlive, Debug
+from common import Options, Internal, isAlive, Debug, handleSignals
+
 
 class MyHook(cgitb.Hook):
 
@@ -57,6 +45,7 @@ class MyHook(cgitb.Hook):
 
 # sys.excepthook = MyHook()
 
+
 NOTFOUND = []
 
 try:
@@ -65,10 +54,6 @@ try:
 except ImportError as importError:
     NOTFOUND.append('Please install PyQt5: %s' % importError)
 
-try:
-    from zope.interface import implements  # pylint: disable=unused-import
-except ImportError as importError:
-    NOTFOUND.append('Package python-zope-interface missing: %s' % importError)
 
 try:
     from twisted.spread import pb # pylint: disable=unused-import
@@ -79,7 +64,7 @@ except ImportError as importError:
 
 try:
     from mi18n import i18n, i18nc
-    from kde import KIcon, KXmlGuiWindow, KStandardAction
+    from kde import KXmlGuiWindow, KStandardAction
 
     from board import FittingView
     from playerlist import PlayerList
@@ -89,23 +74,24 @@ try:
     from scoringdialog import ScoreTable, ExplainView
     from humanclient import HumanClient
     from rulesetselector import RulesetSelector
-    from animation import afterQueuedAnimations, AnimationSpeed, ParallelAnimationGroup
+    from animation import afterQueuedAnimations, AnimationSpeed
     from chat import ChatWindow
     from scene import PlayingScene, ScoringScene
     from configdialog import ConfigDialog
     from statesaver import StateSaver
     from util import checkMemory
+    from kdestub import Action
 
 except ImportError as importError:
     NOTFOUND.append('Kajongg is not correctly installed: modules: %s' %
                     importError)
 
-if len(NOTFOUND):
+if NOTFOUND:
     logError("\n".join(" * %s" % s for s in NOTFOUND), showStack=False)
     sys.exit(3)
 
 
-def cleanExit(*dummyArgs):
+def cleanExit(*unusedArgs): # pylint: disable=unused-argument
     """close sqlite3 files before quitting"""
     if isAlive(Internal.mainWindow):
         if Debug.quit:
@@ -118,13 +104,7 @@ def cleanExit(*dummyArgs):
         # sys.exit(0)
         MainWindow.aboutToQuit()
 
-signal(SIGABRT, cleanExit)
-signal(SIGINT, cleanExit)
-signal(SIGTERM, cleanExit)
-if os.name != 'nt':
-    from signal import SIGHUP, SIGQUIT
-    signal(SIGHUP, cleanExit)
-    signal(SIGQUIT, cleanExit)
+handleSignals(cleanExit)
 
 
 class MainWindow(KXmlGuiWindow):
@@ -134,7 +114,7 @@ class MainWindow(KXmlGuiWindow):
 
     def __init__(self):
         # see https://marc.info/?l=kde-games-devel&m=120071267328984&w=2
-        super(MainWindow, self).__init__()
+        super().__init__()
         Internal.app.aboutToQuit.connect(self.aboutToQuit)
         self.exitConfirmed = None
         self.exitReady = None
@@ -205,7 +185,9 @@ class MainWindow(KXmlGuiWindow):
         self.centralView.setScene(value)
         self.adjustMainView()
         self.updateGUI()
-        self.actionChat.setEnabled(isinstance(value, PlayingScene))
+        canDemo = not value or isinstance(value, PlayingScene)
+        self.actionChat.setEnabled(canDemo)
+        self.actionAutoPlay.setEnabled(canDemo)
         self.actionExplain.setEnabled(value is not None)
         self.actionScoreTable.setEnabled(value is not None)
 
@@ -228,25 +210,9 @@ class MainWindow(KXmlGuiWindow):
         self.centralView.resizeEvent(True)
         KXmlGuiWindow.showEvent(self, event)
 
-    def kajonggAction(
-            self, name, icon, slot=None, shortcut=None, actionData=None):
-        """simplify defining actions"""
-        res = QAction(self)
-        if icon:
-            res.setIcon(KIcon(icon))
-        if slot:
-            res.triggered.connect(slot)
-        self.actionCollection().addAction(name, res)
-        if shortcut:
-            res.setShortcut(Qt.CTRL + shortcut)
-            res.setShortcutContext(Qt.ApplicationShortcut)
-        if actionData is not None:
-            res.setData(actionData)
-        return res
-
     def _kajonggToggleAction(self, name, icon, shortcut=None, actionData=None):
         """a checkable action"""
-        res = self.kajonggAction(
+        res = Action(self,
             name,
             icon,
             shortcut=shortcut,
@@ -276,37 +242,44 @@ class MainWindow(KXmlGuiWindow):
         self.background = None  # just for pylint
         self.windTileset = Tileset(Internal.Preferences.windTilesetName)
         self.adjustMainView()
-        self.actionScoreGame = self.kajonggAction(
+        self.actionScoreGame = Action(
+            self,
             "scoreGame",
             "draw-freehand",
             self.scoringScene,
             Qt.Key_C)
-        self.actionPlayGame = self.kajonggAction(
+        self.actionPlayGame = Action(
+            self,
             "play",
             "arrow-right",
             self.playGame,
             Qt.Key_N)
-        self.actionAbortGame = self.kajonggAction(
+        self.actionAbortGame = Action(
+            self,
             "abort",
             "dialog-close",
             self.abortAction,
             Qt.Key_W)
         self.actionAbortGame.setEnabled(False)
-        self.actionQuit = self.kajonggAction(
+        self.actionQuit = Action(
+            self,
             "quit",
             "application-exit",
             self.close,
             Qt.Key_Q)
-        self.actionPlayers = self.kajonggAction(
+        self.actionPlayers = Action(
+            self,
             "players", "im-user", self.slotPlayers)
-        self.actionRulesets = self.kajonggAction(
+        self.actionRulesets = Action(
+            self,
             "rulesets",
             "games-kajongg-law",
             self.slotRulesets)
         self.actionChat = self._kajonggToggleAction("chat", "call-start",
                                                     shortcut=Qt.Key_H, actionData=ChatWindow)
         self.actionChat.setEnabled(False)
-        self.actionAngle = self.kajonggAction(
+        self.actionAngle = Action(
+            self,
             "angle",
             "object-rotate-left",
             self.changeAngle,
@@ -323,7 +296,8 @@ class MainWindow(KXmlGuiWindow):
         self.actionFullscreen = self._kajonggToggleAction(
             "fullscreen", "view-fullscreen", shortcut=Qt.Key_F + Qt.ShiftModifier)
         self.actionFullscreen.toggled.connect(self.fullScreen)
-        self.actionAutoPlay = self.kajonggAction(
+        self.actionAutoPlay = Action(
+            self,
             "demoMode",
             "arrow-right-double",
             None,
@@ -362,11 +336,11 @@ class MainWindow(KXmlGuiWindow):
         else:
             self.setWindowState(self.windowState() & ~Qt.WindowFullScreen)
 
-    def close(self, dummyResult=None):
+    def close(self, unusedResult=None):
         """wrap close() because we call it with a QTimer"""
         if isAlive(self):
-            ParallelAnimationGroup.cancelAll()
             return KXmlGuiWindow.close(self)
+        return True  # is closed
 
     def closeEvent(self, event):
         KXmlGuiWindow.closeEvent(self, event)
@@ -489,7 +463,7 @@ class MainWindow(KXmlGuiWindow):
         if mainWindow:
             if Debug.quit:
                 logDebug('aboutToQuit starting')
-            if mainWindow.exitWaitTime > 1000.0 or Debug.quit:
+            if mainWindow.exitWaitTime is not None and mainWindow.exitWaitTime > 1000.0 or Debug.quit:
                 logDebug(
                     'reactor stopped after %d ms' %
                     (mainWindow.exitWaitTime))
@@ -613,7 +587,7 @@ class MainWindow(KXmlGuiWindow):
             view.fitInView(scene.itemsBoundingRect(), Qt.KeepAspectRatio)
 
     @afterQueuedAnimations
-    def backgroundChanged(self, dummyDeferredResult, dummyOldName, newName):
+    def backgroundChanged(self, unusedDeferredResult, unusedOldName, newName):
         """if the wanted background changed, apply the change now"""
         centralWidget = self.centralWidget()
         if centralWidget:
@@ -623,7 +597,8 @@ class MainWindow(KXmlGuiWindow):
 
     @afterQueuedAnimations
     def tilesetNameChanged(
-            self, dummyDeferredResult, dummyOldValue=None, dummyNewValue=None, *dummyArgs):
+            self, unusedDeferredResult, unusedOldValue=None,
+            unusedNewValue=None):
         """if the wanted tileset changed, apply the change now"""
         if self.centralView:
             with AnimationSpeed():
@@ -632,7 +607,7 @@ class MainWindow(KXmlGuiWindow):
             self.adjustMainView()
 
     @afterQueuedAnimations
-    def showSettings(self, dummyDeferredResult, dummyChecked=None):
+    def showSettings(self, unusedDeferredResult, unusedChecked=None):
         """show preferences dialog. If it already is visible, do nothing"""
         # This is called by the triggered() signal. So why does KDE
         # not return the bool checked?
@@ -686,7 +661,7 @@ class MainWindow(KXmlGuiWindow):
             scene.updateSceneGUI()
 
     @afterQueuedAnimations
-    def changeAngle(self, deferredResult, dummyButtons=None, dummyModifiers=None): # pylint: disable=unused-argument
+    def changeAngle(self, deferredResult, unusedButtons=None, unusedModifiers=None): # pylint: disable=unused-argument
         """change the lightSource"""
         if self.scene:
             with AnimationSpeed():
